### Summary

* [Booting](Booting/README.md)
    * [From bootloader to kernel](Booting/linux-bootstrap-1.md)
    * [First steps in the kernel setup code](Booting/linux-bootstrap-2.md)
    * [Video mode initialization and transition to protected mode](Booting/linux-bootstrap-3.md)
    * [Transition to 64-bit mode](Booting/linux-bootstrap-4.md)
    * [Kernel decompression](Booting/linux-bootstrap-5.md)
* [Initialization](Initialization/README.md)
    * [First steps in the kernel](Initialization/linux-initialization-1.md)
    * [Early interrupts handler](Initialization/linux-initialization-2.md)
    * [Last preparations before the kernel entry point](Initialization/linux-initialization-3.md)
    * [Kernel entry point](Initialization/linux-initialization-4.md)
    * [Continue architecture-specific boot-time initializations](Initialization/linux-initialization-5.md)
    * [Architecture-specific initializations, again...](Initialization/linux-initialization-6.md)
    * [End of the architecture-specific initializations, almost...](Initialization/linux-initialization-7.md)
    * [Scheduler initialization](Initialization/linux-initialization-8.md)
    * [RCU initialization](Initialization/linux-initialization-9.md)
    * [End of initialization](Initialization/linux-initialization-10.md)
* [Interrupts](interrupts/README.md)
    * [Introduction](interrupts/interrupts-1.md)
    * [Start to dive into interrupts](interrupts/interrupts-2.md)
    * [Interrupt handlers](interrupts/interrupts-3.md)
    * [Initialization of non-early interrupt gates](interrupts/interrupts-4.md)
    * [Implementation of some exception handlers](interrupts/interrupts-5.md)
    * [Handling Non-Maskable interrupts](interrupts/interrupts-6.md)
    * [Dive into external hardware interrupts](interrupts/interrupts-7.md)
    * [Initialization of external hardware interrupts structures](interrupts/interrupts-8.md)
    * [Softirq, Tasklets and Workqueues](interrupts/interrupts-9.md)
    * [Last part](interrupts/interrupts-10.md)
* [System calls](SysCall/README.md)
    * [Introduction to system calls](SysCall/syscall-1.md)
    * [How the Linux kernel handles a system call](SysCall/syscall-2.md)
    * [vsyscall and vDSO](SysCall/syscall-3.md)
    * [How the Linux kernel runs a program](SysCall/syscall-4.md)
* [Timers and time management](Timers/README.md)
    * [Introduction](Timers/timers-1.md)
    * [Clocksource framework](Timers/timers-2.md)
    * [The tick broadcast framework and dyntick](Timers/timers-3.md)
    * [Introduction to timers](Timers/timers-4.md)
    * [Clockevents framework](Timers/timers-5.md)
    * [x86 related clock sources](Timers/timers-6.md)
    * [Time related system calls](Timers/timers-7.md)
* [Synchronization primitives](SyncPrim/README.md)
    * [Introduction to spinlocks](SyncPrim/sync-1.md)
    * [Queued spinlocks](SyncPrim/sync-2.md)
    * [Semaphores](SyncPrim/sync-3.md)
    * [Mutex](SyncPrim/sync-4.md)
    * [Reader/Writer semaphores](SyncPrim/sync-5.md)
    * [SeqLock](SyncPrim/sync-6.md)
    * [RCU]()
    * [Lockdep]()
* [Memory management](mm/README.md)
    * [Memblock](mm/linux-mm-1.md)
    * [Fixmaps and ioremap](mm/linux-mm-2.md)
    * [kmemcheck](mm/mm-3.md)
* [SMP]()
* [Concepts](Concepts/README.md)
    * [Per-CPU variables](Concepts/per-cpu.md)
    * [Cpumasks](Concepts/cpumask.md)
    * [The initcall mechanism](Concepts/initcall.md)
* [Data Structures in the Linux Kernel](DataStructures/README.md)
    * [Doubly linked list](DataStructures/dlist.md)
    * [Radix tree](DataStructures/radix-tree.md)
    * [Bit arrays](DataStructures/bitmap.md)
* [Theory](Theory/README.md)
    * [Paging](Theory/Paging.md)
    * [Elf64](Theory/ELF.md)
    * [Inline assembly](Theory/asm.md)
    * [CPUID]()
    * [MSR]()
* [Initial ram disk]()
   * [initrd]()
* [Misc](Misc/README.md)
    * [How the kernel is compiled](Misc/how_kernel_compiled.md)
    * [Linkers](Misc/linkers.md)
    * [Linux kernel development](Misc/contribute.md)
    * [Write and Submit your first Linux kernel Patch]()
    * [Data types in the kernel]()
* [Useful links](LINKS.md)
* [Contributors](contributors.md)
